from argparse import ArgumentParser

from .root.file_system_root import FilesystemRoot
from .root.dropbox_root import DropboxRoot

class Bash:
    
    def __init__(self, *input):
        self._parser = ArgumentParser()
        self._parser.add_argument('--dropbox', action='store_true')
        known, unknown = self._parser.parse_known_args(input)
        self.commands = unknown
        if known.dropbox:
            self.root = DropboxRoot()
        else:
            self.root = FilesystemRoot()




import http.client
import os

from . import Root

HTTPSConnection = http.client.HTTPSConnection
dropbox_token = os.environ.get('DROPBOX_TOKEN')


class DropboxRoot(Root):

    def __init__(self):
        assert dropbox_token
        super().__init__()

    def _read(self, key):
        c = HTTPSConnection("content.dropboxapi.com")
        h = {
            'Authorization': f"Bearer {dropbox_token}",
            'Dropbox-API-Arg': f'{{"path": "/{key}.txt"}}'
        }
        c.request("POST", "/2/files/download", headers=h)
        r = c.getresponse()
        if r.status == 200:
            return [i.strip() for i in r.read().decode("utf-8").split('\n')]

    def _save(self, key, items):
        c = HTTPSConnection("content.dropboxapi.com")
        h = {
            'Authorization': f"Bearer {dropbox_token}",
            'Dropbox-API-Arg': f'{{"path": "/{key}.txt"}}',
            'Content-Type': 'application/octet-stream'
        }
        body = "\n".join(str(i) for i in items)
        c.request("POST", "/2/files/upload", headers=h, body=body)




from ..queue import Queue


class Root:

    def __init__(self):
        self._queues = {}

    def _read(self, key):
        return ""

    def get_queue(self, key=None):
        key = key or Queue.default_key
        queueclass = Queue.subclass(key)
        lines = self._read(key)
        self._queues[key] = queueclass(self, lines)
        return self._queues[key]

    def _save(self, key, items):  # pragma: no cover
        pass

    def save(self):
        while self._queues:
            key, queue = self._queues.popitem()
            if queue.changed:
                items = queue.all()
                self._save(key, items)



from unittest import TestCase
from tempfile import TemporaryDirectory
from pathlib import Path
from unittest import mock
from unittest.mock import MagicMock

from busy.plugins.todo import TodoQueue
from busy.root.file_system_root import FilesystemRoot
from busy.file import File
from busy.root.dropbox_root import DropboxRoot

class TestFilesystemRoot(TestCase):

    def test_root(self):
        with TemporaryDirectory() as d:
            sd = FilesystemRoot(Path(d))
            s = sd.get_queue('tasks')
            self.assertIsInstance(s, TodoQueue)

    def test_add_todo(self):
        with TemporaryDirectory() as td:
            sd1 = FilesystemRoot(Path(td))
            sd1.get_queue('x').add('a')
            sd1.save()
            sd2 = FilesystemRoot(Path(td))
            self.assertEqual(str(sd2.get_queue('x').get()),'a')

    def test_make_dir_pater(self):
        with TemporaryDirectory() as td:
            r = FilesystemRoot(Path(td))
            r.get_queue('y').add('a')
            r.save()
            r2 = FilesystemRoot(Path(td))
            self.assertEqual(str(r2.get_queue('y').get()),'a')

    def test_env_var_as_backup(self):
        with TemporaryDirectory() as td:
            with mock.patch.dict('os.environ', {'BUSY_ROOT': td}):
                sd1 = FilesystemRoot()
                sd1.get_queue('p').add('a')
                sd1.save()
                f = Path(td) / 'p.txt'
                self.assertEqual(f.read_text(),'a\n')

    def test_user_root(self):
        with TemporaryDirectory() as td:
            with mock.patch.dict('os.environ', clear=True):
                with mock.patch('pathlib.Path.home', lambda : Path(td)):
                    sd1 = FilesystemRoot()
                    sd1.get_queue('w').add('a')
                    sd1.save()
                    f = Path(td) / '.busy' / 'w.txt'
                    self.assertEqual(f.read_text(),'a\n')


class MockResponse():
    
    def __init__(self):
        self.status = 200

MockResponse.read = MagicMock(return_value=bytearray("2019-01-01|a",'utf-8'))


class MockConnection():

    def __init__(self, domain):
        pass

MockConnection.request = MagicMock()
MockConnection.getresponse = MagicMock(return_value=MockResponse())


class TestDropboxRoot(TestCase):

    def test_mock(self):
        c = MockConnection("")
        c.request("", "", headers={})
        r = c.getresponse()
        self.assertEqual(r.read().decode('utf-8'), '2019-01-01|a')
    
    def test_get_queue_outline(self):
        m = MockConnection
        with mock.patch('busy.root.dropbox_root.HTTPSConnection', m):
            with mock.patch('busy.root.dropbox_root.dropbox_token','x'):
                sd = DropboxRoot()
                s = sd.get_queue('tasks')
                t = s.get()
                self.assertEqual(str(t), '2019-01-01|a')
                m.request.assert_called_with('POST', '/2/files/download', headers={'Authorization': 'Bearer x', 'Dropbox-API-Arg': '{"path": "/tasks.txt"}'})

    def test_get_another_queue(self):
        m = MockConnection
        with mock.patch('busy.root.dropbox_root.HTTPSConnection', m):
            with mock.patch('busy.root.dropbox_root.dropbox_token','x'):
                sd = DropboxRoot()
                s = sd.get_queue('z')
                s.get()
                m.request.assert_called_with('POST', '/2/files/download', headers={'Authorization': 'Bearer x', 'Dropbox-API-Arg': '{"path": "/z.txt"}'})

    def test_save(self):
        m = MockConnection
        with mock.patch('busy.root.dropbox_root.HTTPSConnection', m):
            with mock.patch('busy.root.dropbox_root.dropbox_token','x'):
                sd = DropboxRoot()
                s = sd.get_queue('z')
                s.add('a')
                sd.save()
                m.request.assert_called_with('POST','/2/files/upload', body='2019-01-01|a\na', headers={'Authorization': 'Bearer x', 'Dropbox-API-Arg': '{"path": "/z.txt"}', 'Content-Type': 'application/octet-stream'})



from unittest import TestCase
from unittest import mock

from busy.bash import Bash
from busy.root.file_system_root import FilesystemRoot
from busy.root.dropbox_root import DropboxRoot

class TestBash(TestCase):

    def test_commands(self):
        b = Bash('a','b')
        self.assertEqual(b.commands, ['a','b'])

    def test_default_storage(self):
        b = Bash('a','b')
        self.assertIsInstance(b.root, FilesystemRoot)

    def test_dropbox_storage_commands(self):
        with mock.patch('busy.root.dropbox_root.dropbox_token','x'):
            b = Bash('--dropbox','a','b')
            self.assertEqual(b.commands, ['a','b'])

    def test_dropbox_storage_root(self):
        with mock.patch('busy.root.dropbox_root.dropbox_token','x'):
            b = Bash('--dropbox','a','b')
            self.assertIsInstance(b.root, DropboxRoot)

    def test_optional_args_persist(self):
        with mock.patch('busy.root.dropbox_root.dropbox_token','x'):
            b = Bash('--dropbox','a','--b','c')
            self.assertEqual(b.commands, ['a', '--b', 'c'])



from pathlib import Path
from tempfile import TemporaryDirectory
import os

from ..file import File
from ..queue import Queue
from . import Root


class FilesystemRoot(Root):

    def __init__(self, path=None):
        if path:
            self._path = Path(path) if isinstance(path, str) else path
            assert isinstance(self._path, Path) and self._path.is_dir()
        else:
            env_var = os.environ.get('BUSY_ROOT')
            self._path = Path(env_var if env_var else Path.home() / '.busy')
            if not self._path.is_dir():
                self._path.mkdir()
        self._files = {}
        self._queues = {}

    def get_queue(self, key=None):
        key = key or Queue.default_key
        if key not in self._queues:
            queueclass = Queue.subclass(key)
            queuefile = File(self._path / f'{key}.txt')
            self._files[key] = queuefile
            self._queues[key] = queueclass(self)
            self._queues[key].add(*queuefile.read(queueclass.itemclass))
        return self._queues[key]

    def _save(self, key, items):
        self._files[key].save(*items)
